from __future__ import absolute_import, unicode_literals

import os

from mopidy.internal import deprecation


def path_to_data_dir(name):
    if not isinstance(name, bytes):
        name = name.encode('utf-8')
    path = os.path.dirname(__file__)
    path = os.path.join(path, b'data')
    path = os.path.abspath(path)
    return os.path.join(path, name)


def generate_song(i):
    return 'local:track:song%s.wav' % i


def populate_tracklist(func):
    def wrapper(self):
        with deprecation.ignore('core.tracklist.add:tracks_arg'):
            self.tl_tracks = self.core.tracklist.add(self.tracks)
        return func(self)

    wrapper.__name__ = func.__name__
    wrapper.__doc__ = func.__doc__
    return wrapper



from __future__ import absolute_import, absolute_import, unicode_literals

import collections
import logging
import os
import re
import sys

import mopidy
from mopidy import compat, models
from mopidy.internal import storage as internal_storage
from mopidy.internal import timer

from mopidy_local import Extension, Library, search, storage, translator


logger = logging.getLogger(__name__)


class _BrowseCache(object):
    encoding = sys.getfilesystemencoding()
    splitpath_re = re.compile(r'([^/]+)')

    def __init__(self, uris):
        self._cache = {
            Library.ROOT_DIRECTORY_URI: collections.OrderedDict()}

        for track_uri in uris:
            path = translator.local_track_uri_to_path(track_uri, b'/')
            parts = self.splitpath_re.findall(
                path.decode(self.encoding, 'replace'))
            track_ref = models.Ref.track(uri=track_uri, name=parts.pop())

            # Look for our parents backwards as this is faster than having to
            # do a complete search for each add.
            parent_uri = None
            child = None
            for i in reversed(range(len(parts))):
                directory = '/'.join(parts[:i + 1])
                uri = translator.path_to_local_directory_uri(directory)

                # First dir we process is our parent
                if not parent_uri:
                    parent_uri = uri

                # We found ourselves and we exist, done.
                if uri in self._cache:
                    if child:
                        self._cache[uri][child.uri] = child
                    break

                # Initialize ourselves, store child if present, and add
                # ourselves as child for next loop.
                self._cache[uri] = collections.OrderedDict()
                if child:
                    self._cache[uri][child.uri] = child
                child = models.Ref.directory(uri=uri, name=parts[i])
            else:
                # Loop completed, so final child needs to be added to root.
                if child:
                    self._cache[Library.ROOT_DIRECTORY_URI][child.uri] = child
                # If no parent was set we belong in the root.
                if not parent_uri:
                    parent_uri = Library.ROOT_DIRECTORY_URI

            self._cache[parent_uri][track_uri] = track_ref

    def lookup(self, uri):
        return self._cache.get(uri, {}).values()


class JsonLibrary(Library):
    name = 'json'

    def __init__(self, config):
        self._tracks = {}
        self._browse_cache = None
        self._media_dir = config['local']['media_dir']
        self._json_file = os.path.join(
            Extension.get_data_dir(config), b'library.json.gz')

        storage.check_dirs_and_files(config)

    def browse(self, uri):
        if not self._browse_cache:
            return []
        return self._browse_cache.lookup(uri)

    def load(self):
        logger.debug('Loading library: %s', self._json_file)
        with timer.time_logger('Loading tracks'):
            if not os.path.isfile(self._json_file):
                logger.info(
                    'No local library metadata cache found at %s. Please run '
                    '`mopidy local scan` to index your local music library. '
                    'If you do not have a local music collection, you can '
                    'disable the local backend to hide this message.',
                    self._json_file)
                self._tracks = {}
            else:
                library = internal_storage.load(self._json_file)
                self._tracks = {
                    t.uri: t for t in library.get('tracks', [])}
        with timer.time_logger('Building browse cache'):
            self._browse_cache = _BrowseCache(sorted(self._tracks.keys()))
        return len(self._tracks)

    def lookup(self, uri):
        try:
            return [self._tracks[uri]]
        except KeyError:
            return []

    def get_distinct(self, field, query=None):
        if field == 'track':
            def distinct(track):
                return {track.name}
        elif field == 'artist':
            def distinct(track):
                return {a.name for a in track.artists}
        elif field == 'albumartist':
            def distinct(track):
                album = track.album or models.Album()
                return {a.name for a in album.artists}
        elif field == 'album':
            def distinct(track):
                album = track.album or models.Album()
                return {album.name}
        elif field == 'composer':
            def distinct(track):
                return {a.name for a in track.composers}
        elif field == 'performer':
            def distinct(track):
                return {a.name for a in track.performers}
        elif field == 'date':
            def distinct(track):
                return {track.date}
        elif field == 'genre':
            def distinct(track):
                return {track.genre}
        else:
            return set()

        distinct_result = set()
        search_result = search.search(self._tracks.values(), query, limit=None)
        for track in search_result.tracks:
            distinct_result.update(distinct(track))
        return distinct_result - {None}

    def search(self, query=None, limit=100, offset=0, uris=None, exact=False):
        tracks = self._tracks.values()
        if exact:
            return search.find_exact(
                tracks, query=query, limit=limit, offset=offset, uris=uris)
        else:
            return search.search(
                tracks, query=query, limit=limit, offset=offset, uris=uris)

    def begin(self):
        return compat.itervalues(self._tracks)

    def add(self, track):
        self._tracks[track.uri] = track

    def remove(self, uri):
        self._tracks.pop(uri, None)

    def close(self):
        internal_storage.dump(self._json_file, {
            'version': mopidy.__version__,
            'tracks': self._tracks.values()
        })

    def clear(self):
        try:
            os.remove(self._json_file)
            return True
        except OSError:
            return False



from __future__ import absolute_import, unicode_literals

import logging
import os

logger = logging.getLogger(__name__)


def check_dirs_and_files(config):
    if not os.path.isdir(config['local']['media_dir']):
        logger.warning(
            'Local media dir %s does not exist or we lack permissions to the '
            'directory or one of its parents' % config['local']['media_dir'])



from __future__ import absolute_import, unicode_literals

import logging

from mopidy import backend, models

from mopidy_local import Library


logger = logging.getLogger(__name__)


class LocalLibraryProvider(backend.LibraryProvider):

    """Proxy library that delegates work to our active local library."""

    root_directory = models.Ref.directory(
        uri=Library.ROOT_DIRECTORY_URI, name='Local media')

    def __init__(self, backend, library):
        super(LocalLibraryProvider, self).__init__(backend)
        self._library = library
        self.refresh()

    def browse(self, uri):
        if not self._library:
            return []
        return self._library.browse(uri)

    def get_distinct(self, field, query=None):
        if not self._library:
            return set()
        return self._library.get_distinct(field, query)

    def get_images(self, uris):
        if not self._library:
            return {}
        return self._library.get_images(uris)

    def refresh(self, uri=None):
        if not self._library:
            return 0
        num_tracks = self._library.load()
        logger.info('Loaded %d local tracks using %s',
                    num_tracks, self._library.name)

    def lookup(self, uri):
        if not self._library:
            return []
        tracks = self._library.lookup(uri)
        if tracks is None:
            logger.debug('Failed to lookup %r', uri)
            return []
        if isinstance(tracks, models.Track):
            tracks = [tracks]
        return tracks

    def search(self, query=None, uris=None, exact=False):
        if not self._library:
            return None
        return self._library.search(query=query, uris=uris, exact=exact)



from __future__ import absolute_import, unicode_literals

import random
import unittest

from mopidy import core
from mopidy.core import PlaybackState
from mopidy.models import Playlist, Track

import pykka

from mopidy_local import actor

from tests import (
    dummy_audio, generate_song, path_to_data_dir, populate_tracklist)


class LocalTracklistProviderTest(unittest.TestCase):
    config = {
        'core': {
            'data_dir': path_to_data_dir(''),
            'max_tracklist_length': 10000
        },
        'local': {
            'media_dir': path_to_data_dir(''),
            'playlists_dir': b'',
            'library': 'json',
        }
    }
    tracks = [
        Track(uri=generate_song(i), length=4464) for i in range(1, 4)]

    def setUp(self):  # noqa: N802
        self.audio = dummy_audio.create_proxy()
        self.backend = actor.LocalBackend.start(
            config=self.config, audio=self.audio).proxy()
        self.core = core.Core.start(audio=self.audio,
                                    backends=[self.backend],
                                    config=self.config).proxy()
        self.controller = self.core.tracklist
        self.playback = self.core.playback

        assert len(self.tracks) == 3, 'Need three tracks to run tests.'

    def tearDown(self):  # noqa: N802
        pykka.ActorRegistry.stop_all()

    def assert_state_is(self, state):
        self.assertEqual(self.playback.get_state().get(), state)

    def assert_current_track_is(self, track):
        self.assertEqual(self.playback.get_current_track().get(), track)

    def test_length(self):
        self.assertEqual(0, len(self.controller.get_tl_tracks().get()))
        self.assertEqual(0, self.controller.get_length().get())
        self.controller.add(self.tracks)
        self.assertEqual(3, len(self.controller.get_tl_tracks().get()))
        self.assertEqual(3, self.controller.get_length().get())

    def test_add(self):
        for track in self.tracks:
            added = self.controller.add([track]).get()
            tracks = self.controller.get_tracks().get()
            tl_tracks = self.controller.get_tl_tracks().get()

            self.assertEqual(track, tracks[-1])
            self.assertEqual(added[0], tl_tracks[-1])
            self.assertEqual(track, added[0].track)

    def test_add_at_position(self):
        for track in self.tracks[:-1]:
            added = self.controller.add([track], 0).get()
            tracks = self.controller.get_tracks().get()
            tl_tracks = self.controller.get_tl_tracks().get()

            self.assertEqual(track, tracks[0])
            self.assertEqual(added[0], tl_tracks[0])
            self.assertEqual(track, added[0].track)

    @populate_tracklist
    def test_add_at_position_outside_of_playlist(self):
        for track in self.tracks:
            added = self.controller.add([track], len(self.tracks) + 2).get()
            tracks = self.controller.get_tracks().get()
            tl_tracks = self.controller.get_tl_tracks().get()

            self.assertEqual(track, tracks[-1])
            self.assertEqual(added[0], tl_tracks[-1])
            self.assertEqual(track, added[0].track)

    @populate_tracklist
    def test_filter_by_tlid(self):
        tl_track = self.controller.get_tl_tracks().get()[1]
        result = self.controller.filter({'tlid': [tl_track.tlid]}).get()
        self.assertEqual([tl_track], result)

    @populate_tracklist
    def test_filter_by_uri(self):
        tl_track = self.controller.get_tl_tracks().get()[1]
        result = self.controller.filter({'uri': [tl_track.track.uri]}).get()
        self.assertEqual([tl_track], result)

    @populate_tracklist
    def test_filter_by_uri_returns_nothing_for_invalid_uri(self):
        self.assertEqual([], self.controller.filter({'uri': ['foobar']}).get())

    def test_filter_by_uri_returns_single_match(self):
        t = Track(uri='a')
        self.controller.add([Track(uri='z'), t, Track(uri='y')])

        result = self.controller.filter({'uri': ['a']}).get()
        self.assertEqual(t, result[0].track)

    def test_filter_by_uri_returns_multiple_matches(self):
        track = Track(uri='a')
        self.controller.add([Track(uri='z'), track, track])
        tl_tracks = self.controller.filter({'uri': ['a']}).get()
        self.assertEqual(track, tl_tracks[0].track)
        self.assertEqual(track, tl_tracks[1].track)

    def test_filter_by_uri_returns_nothing_if_no_match(self):
        self.controller.playlist = Playlist(
            tracks=[Track(uri='z'), Track(uri='y')])
        self.assertEqual([], self.controller.filter({'uri': ['a']}).get())

    def test_filter_by_multiple_criteria_returns_elements_matching_all(self):
        t1 = Track(uri='a', name='x')
        t2 = Track(uri='b', name='x')
        t3 = Track(uri='b', name='y')
        self.controller.add([t1, t2, t3])

        result1 = self.controller.filter({'uri': ['a'], 'name': ['x']}).get()
        self.assertEqual(t1, result1[0].track)

        result2 = self.controller.filter({'uri': ['b'], 'name': ['x']}).get()
        self.assertEqual(t2, result2[0].track)

        result3 = self.controller.filter({'uri': ['b'], 'name': ['y']}).get()
        self.assertEqual(t3, result3[0].track)

    def test_filter_by_criteria_that_is_not_present_in_all_elements(self):
        track1 = Track()
        track2 = Track(uri='b')
        track3 = Track()

        self.controller.add([track1, track2, track3])
        result = self.controller.filter({'uri': ['b']}).get()
        self.assertEqual(track2, result[0].track)

    @populate_tracklist
    def test_clear(self):
        self.controller.clear().get()
        self.assertEqual(len(self.controller.get_tracks().get()), 0)

    def test_clear_empty_playlist(self):
        self.controller.clear().get()
        self.assertEqual(len(self.controller.get_tracks().get()), 0)

    @populate_tracklist
    def test_clear_when_playing(self):
        self.playback.play().get()
        self.assert_state_is(PlaybackState.PLAYING)
        self.controller.clear().get()
        self.assert_state_is(PlaybackState.STOPPED)

    def test_add_appends_to_the_tracklist(self):
        self.controller.add([Track(uri='a'), Track(uri='b')])

        tracks = self.controller.get_tracks().get()
        self.assertEqual(len(tracks), 2)

        self.controller.add([Track(uri='c'), Track(uri='d')])

        tracks = self.controller.get_tracks().get()
        self.assertEqual(len(tracks), 4)
        self.assertEqual(tracks[0].uri, 'a')
        self.assertEqual(tracks[1].uri, 'b')
        self.assertEqual(tracks[2].uri, 'c')
        self.assertEqual(tracks[3].uri, 'd')

    def test_add_does_not_reset_version(self):
        version = self.controller.get_version().get()
        self.controller.add([])
        self.assertEqual(self.controller.get_version().get(), version)

    @populate_tracklist
    def test_add_preserves_playing_state(self):
        self.playback.play().get()

        track = self.playback.get_current_track().get()
        tracks = self.controller.get_tracks().get()
        self.controller.add(tracks[1:2]).get()

        self.assert_state_is(PlaybackState.PLAYING)
        self.assert_current_track_is(track)

    @populate_tracklist
    def test_add_preserves_stopped_state(self):
        tracks = self.controller.get_tracks().get()
        self.controller.add(tracks[1:2]).get()

        self.assert_state_is(PlaybackState.STOPPED)
        self.assert_current_track_is(None)

    @populate_tracklist
    def test_add_returns_the_tl_tracks_that_was_added(self):
        tracks = self.controller.get_tracks().get()

        added = self.controller.add(tracks[1:2]).get()
        tracks = self.controller.get_tracks().get()
        self.assertEqual(added[0].track, tracks[1])

    @populate_tracklist
    def test_move_single(self):
        self.controller.move(0, 0, 2)

        tracks = self.controller.get_tracks().get()
        self.assertEqual(tracks[2], self.tracks[0])

    @populate_tracklist
    def test_move_group(self):
        self.controller.move(0, 2, 1)

        tracks = self.controller.get_tracks().get()
        self.assertEqual(tracks[1], self.tracks[0])
        self.assertEqual(tracks[2], self.tracks[1])

    @populate_tracklist
    def test_moving_track_outside_of_playlist(self):
        num_tracks = len(self.controller.get_tracks().get())
        with self.assertRaises(AssertionError):
            self.controller.move(0, 0, num_tracks + 5).get()

    @populate_tracklist
    def test_move_group_outside_of_playlist(self):
        num_tracks = len(self.controller.get_tracks().get())
        with self.assertRaises(AssertionError):
            self.controller.move(0, 2, num_tracks + 5).get()

    @populate_tracklist
    def test_move_group_out_of_range(self):
        num_tracks = len(self.controller.get_tracks().get())
        with self.assertRaises(AssertionError):
            self.controller.move(num_tracks + 2, num_tracks + 3, 0).get()

    @populate_tracklist
    def test_move_group_invalid_group(self):
        with self.assertRaises(AssertionError):
            self.controller.move(2, 1, 0).get()

    def test_tracks_attribute_is_immutable(self):
        tracks1 = self.controller.get_tracks().get()
        tracks2 = self.controller.get_tracks().get()
        self.assertNotEqual(id(tracks1), id(tracks2))

    @populate_tracklist
    def test_remove(self):
        track1 = self.controller.get_tracks().get()[1]
        track2 = self.controller.get_tracks().get()[2]
        version = self.controller.get_version().get()
        self.controller.remove({'uri': [track1.uri]})
        self.assertLess(version, self.controller.get_version().get())
        self.assertNotIn(track1, self.controller.get_tracks().get())
        self.assertEqual(track2, self.controller.get_tracks().get()[1])

    @populate_tracklist
    def test_removing_track_that_does_not_exist_does_nothing(self):
        self.controller.remove({'uri': ['/nonexistant']}).get()

    def test_removing_from_empty_playlist_does_nothing(self):
        self.controller.remove({'uri': ['/nonexistant']}).get()

    @populate_tracklist
    def test_remove_lists(self):
        version = self.controller.get_version().get()
        tracks = self.controller.get_tracks().get()
        track0 = tracks[0]
        track1 = tracks[1]
        track2 = tracks[2]

        self.controller.remove({'uri': [track0.uri, track2.uri]})

        tracks = self.controller.get_tracks().get()
        self.assertLess(version, self.controller.get_version().get())
        self.assertNotIn(track0, tracks)
        self.assertNotIn(track2, tracks)
        self.assertEqual(track1, tracks[0])

    @populate_tracklist
    def test_shuffle(self):
        random.seed(1)
        self.controller.shuffle()

        shuffled_tracks = self.controller.get_tracks().get()

        self.assertNotEqual(self.tracks, shuffled_tracks)
        self.assertEqual(set(self.tracks), set(shuffled_tracks))

    @populate_tracklist
    def test_shuffle_subset(self):
        random.seed(1)
        self.controller.shuffle(1, 3)

        shuffled_tracks = self.controller.get_tracks().get()

        self.assertNotEqual(self.tracks, shuffled_tracks)
        self.assertEqual(self.tracks[0], shuffled_tracks[0])
        self.assertEqual(set(self.tracks), set(shuffled_tracks))

    @populate_tracklist
    def test_shuffle_invalid_subset(self):
        with self.assertRaises(AssertionError):
            self.controller.shuffle(3, 1).get()

    @populate_tracklist
    def test_shuffle_superset(self):
        num_tracks = len(self.controller.get_tracks().get())
        with self.assertRaises(AssertionError):
            self.controller.shuffle(1, num_tracks + 5).get()

    @populate_tracklist
    def test_shuffle_open_subset(self):
        random.seed(1)
        self.controller.shuffle(1)

        shuffled_tracks = self.controller.get_tracks().get()

        self.assertNotEqual(self.tracks, shuffled_tracks)
        self.assertEqual(self.tracks[0], shuffled_tracks[0])
        self.assertEqual(set(self.tracks), set(shuffled_tracks))

    @populate_tracklist
    def test_slice_returns_a_subset_of_tracks(self):
        track_slice = self.controller.slice(1, 3).get()
        self.assertEqual(2, len(track_slice))
        self.assertEqual(self.tracks[1], track_slice[0].track)
        self.assertEqual(self.tracks[2], track_slice[1].track)

    @populate_tracklist
    def test_slice_returns_empty_list_if_indexes_outside_tracks_list(self):
        self.assertEqual(0, len(self.controller.slice(7, 8).get()))
        self.assertEqual(0, len(self.controller.slice(-1, 1).get()))

    def test_version_does_not_change_when_adding_nothing(self):
        version = self.controller.get_version().get()
        self.controller.add([])
        self.assertEqual(version, self.controller.get_version().get())

    def test_version_increases_when_adding_something(self):
        version = self.controller.get_version().get()
        self.controller.add([Track()])
        self.assertLess(version, self.controller.get_version().get())



from __future__ import absolute_import, unicode_literals

import logging
import os
import urllib

from mopidy import compat
from mopidy.internal import path


logger = logging.getLogger(__name__)


def local_uri_to_file_uri(uri, media_dir):
    """Convert local track or directory URI to file URI."""
    return path_to_file_uri(local_uri_to_path(uri, media_dir))


def local_uri_to_path(uri, media_dir):
    """Convert local track or directory URI to absolute path."""
    if (
            not uri.startswith('local:directory:') and
            not uri.startswith('local:track:')):
        raise ValueError('Invalid URI.')
    file_path = path.uri_to_path(uri).split(b':', 1)[1]
    return os.path.join(media_dir, file_path)


def local_track_uri_to_path(uri, media_dir):
    # Deprecated version to keep old versions of Mopidy-Local-Sqlite working.
    return local_uri_to_path(uri, media_dir)


def path_to_file_uri(abspath):
    """Convert absolute path to file URI."""
    # Re-export internal method for use by Mopidy-Local-* extensions.
    return path.path_to_uri(abspath)


def path_to_local_track_uri(relpath):
    """Convert path relative to :confval:`local/media_dir` to local track
    URI."""
    if isinstance(relpath, compat.text_type):
        relpath = relpath.encode('utf-8')
    return 'local:track:%s' % urllib.quote(relpath)


def path_to_local_directory_uri(relpath):
    """Convert path relative to :confval:`local/media_dir` to directory URI."""
    if isinstance(relpath, compat.text_type):
        relpath = relpath.encode('utf-8')
    return 'local:directory:%s' % urllib.quote(relpath)



# encoding: utf-8

from __future__ import unicode_literals

from mopidy import compat

import pytest

from mopidy_local import translator


@pytest.mark.parametrize('local_uri,file_uri', [
    ('local:directory:A/B', 'file:///home/alice/Music/A/B'),
    ('local:directory:A%20B', 'file:///home/alice/Music/A%20B'),
    ('local:directory:A+B', 'file:///home/alice/Music/A%2BB'),
    (
        'local:directory:%C3%A6%C3%B8%C3%A5',
        'file:///home/alice/Music/%C3%A6%C3%B8%C3%A5'),
    ('local:track:A/B.mp3', 'file:///home/alice/Music/A/B.mp3'),
    ('local:track:A%20B.mp3', 'file:///home/alice/Music/A%20B.mp3'),
    ('local:track:A+B.mp3', 'file:///home/alice/Music/A%2BB.mp3'),
    (
        'local:track:%C3%A6%C3%B8%C3%A5.mp3',
        'file:///home/alice/Music/%C3%A6%C3%B8%C3%A5.mp3'),
])
def test_local_uri_to_file_uri(local_uri, file_uri):
    media_dir = b'/home/alice/Music'

    assert translator.local_uri_to_file_uri(local_uri, media_dir) == file_uri


@pytest.mark.parametrize('uri', [
    'A/B',
    'local:foo:A/B',
])
def test_local_uri_to_file_uri_errors(uri):
    media_dir = b'/home/alice/Music'

    with pytest.raises(ValueError):
        translator.local_uri_to_file_uri(uri, media_dir)


@pytest.mark.parametrize('uri,path', [
    ('local:directory:A/B', b'/home/alice/Music/A/B'),
    ('local:directory:A%20B', b'/home/alice/Music/A B'),
    ('local:directory:A+B', b'/home/alice/Music/A+B'),
    (
        'local:directory:%C3%A6%C3%B8%C3%A5',
        b'/home/alice/Music/\xc3\xa6\xc3\xb8\xc3\xa5'),
    ('local:track:A/B.mp3', b'/home/alice/Music/A/B.mp3'),
    ('local:track:A%20B.mp3', b'/home/alice/Music/A B.mp3'),
    ('local:track:A+B.mp3', b'/home/alice/Music/A+B.mp3'),
    (
        'local:track:%C3%A6%C3%B8%C3%A5.mp3',
        b'/home/alice/Music/\xc3\xa6\xc3\xb8\xc3\xa5.mp3'),
])
def test_local_uri_to_path(uri, path):
    media_dir = b'/home/alice/Music'

    assert translator.local_uri_to_path(uri, media_dir) == path

    # Legacy version to keep old versions of Mopidy-Local-Sqlite working
    assert translator.local_track_uri_to_path(uri, media_dir) == path


@pytest.mark.parametrize('uri', [
    'A/B',
    'local:foo:A/B',
])
def test_local_uri_to_path_errors(uri):
    media_dir = b'/home/alice/Music'

    with pytest.raises(ValueError):
        translator.local_uri_to_path(uri, media_dir)


@pytest.mark.parametrize('path,uri', [
    ('/foo', 'file:///foo'),
    (b'/foo', 'file:///foo'),
    ('/æøå', 'file:///%C3%A6%C3%B8%C3%A5'),
    (b'/\x00\x01\x02', 'file:///%00%01%02'),
])
def test_path_to_file_uri(path, uri):
    assert translator.path_to_file_uri(path) == uri


@pytest.mark.parametrize('path,uri', [
    ('foo', 'local:track:foo'),
    (b'foo', 'local:track:foo'),
    ('æøå', 'local:track:%C3%A6%C3%B8%C3%A5'),
    (b'\x00\x01\x02', 'local:track:%00%01%02'),
])
def test_path_to_local_track_uri(path, uri):
    result = translator.path_to_local_track_uri(path)
    assert isinstance(result, compat.text_type)
    assert result == uri


@pytest.mark.parametrize('path,uri', [
    ('foo', 'local:directory:foo'),
    (b'foo', 'local:directory:foo'),
    ('æøå', 'local:directory:%C3%A6%C3%B8%C3%A5'),
    (b'\x00\x01\x02', 'local:directory:%00%01%02'),
])
def test_path_to_local_directory_uri(path, uri):
    result = translator.path_to_local_directory_uri(path)
    assert isinstance(result, compat.text_type)
    assert result == uri



from __future__ import unicode_literals

import re

from setuptools import find_packages, setup


def get_version(filename):
    with open(filename) as fh:
        metadata = dict(re.findall('__([a-z]+)__ = "([^"]+)"', fh.read()))
        return metadata["version"]


setup(
    name="Mopidy-Local",
    version=get_version("mopidy_local/__init__.py"),
    url="https://github.com/mopidy/mopidy-local",
    license="Apache License, Version 2.0",
    author="Stein Magnus Jodal",
    author_email="stein.magnus@jodal.no",
    description=(
        "Mopidy extension for playing music from your local music archive"),
    long_description=open("README.rst").read(),
    packages=find_packages(exclude=["tests", "tests.*"]),
    zip_safe=False,
    include_package_data=True,
    python_requires="> 2.7, < 3",
    install_requires=[
        "setuptools",
        "Mopidy >= 3.0.0a1",
        "Pykka >= 2.0",
    ],
    entry_points={
        "mopidy.ext": [
            "local = mopidy_local:Extension",
        ]
    },
    classifiers=[
        "Environment :: No Input/Output (Daemon)",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Topic :: Multimedia :: Sound/Audio :: Players",
    ],
)



from __future__ import unicode_literals

import unittest

from mopidy.models import Album, Track

from mopidy_local import search


class LocalLibrarySearchTest(unittest.TestCase):

    def test_find_exact_with_album_query(self):
        expected_tracks = [Track(album=Album(name='foo'))]
        tracks = [Track(), Track(album=Album(name='bar'))] + expected_tracks

        search_result = search.find_exact(tracks, {'album': ['foo']})

        self.assertEqual(search_result.tracks, tuple(expected_tracks))
